<?php

namespace Lc\SovBundle\Definition\Field;

use EasyCorp\Bundle\EasyAdminBundle\Config\Crud;
use EasyCorp\Bundle\EasyAdminBundle\Field\FormField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextareaField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;
use Lc\SovBundle\Field\CollectionField;
use Lc\SovBundle\Translation\TranslatorAdmin;
use PhpParser\Node\Expr\Throw_;
use Symfony\Component\Form\Extension\Core\Type\TextType;

abstract class AbstractFieldDefinition
{
    protected TranslatorAdmin $translatorAdmin;

    public function __construct(TranslatorAdmin $translatorAdmin)
    {
        $this->translatorAdmin = $translatorAdmin;
    }

    abstract public function configureFields(): array;

    public function configureFieldsBase(): array
    {
        return [
                'metaTitle' => TextField::new('metaTitle')
                        ->setLabel('Meta Title')
                        ->setHelp('Affiché dans les résultats de recherche Google')
                        ->hideOnIndex(),
                'metaDescription' => TextareaField::new('metaDescription')
                        ->setLabel('Meta description')
                        ->setHelp('Affiché dans les résultats de recherche Google')
                        ->hideOnIndex(),
                'oldUrls' => CollectionField::new('oldUrls')
                        ->setFormTypeOption('entry_type', TextType::class)
                        ->setLabel('Anciennes urls du document')
                        ->hideOnIndex(),
                'devAlias' => TextField::new('devAlias')->hideOnIndex(),
        ];
    }

    public function configureDefaultFields(): array
    {
        return array_keys($this->configureFields());
    }

    public function configureIndex(): array
    {
        return $this->configureDefaultFields();
    }

    public function configureDetail(): array
    {
        return $this->configureDefaultFields();
    }

    public function configureForm(): array
    {
        return $this->configureDefaultFields();
    }

    public function configurePanels(): array
    {
        return [];
    }

    public function configurePanelSeo(): array
    {
        return [
                'metaTitle',
                'metaDescription',
                'oldUrls'
        ];
    }

    public function configurePanelConf(): array
    {
        return [
                'devAlias',
        ];
    }

    public function getFields(string $pageName = ''): array
    {
        if($pageName == Crud::PAGE_INDEX) {
            return $this->buildFieldArray($this->configureIndex());
        }
        elseif($pageName == Crud::PAGE_DETAIL) {
            return $this->buildFieldArray($this->configureDetail());
        }
        elseif($pageName == Crud::PAGE_EDIT || $pageName == Crud::PAGE_NEW) {
            return $this->buildFieldArrayForm();
        }

        return [];
    }

    public function getAllFields(): array
    {
        return array_merge(
                $this->configureFieldsBase(),
                $this->configureFields()
        );
    }

    public function buildFieldArray(array $configureFieldArray): array
    {
        $allFieldArray = $this->getAllFields();

        $fieldArray = [];
        foreach($configureFieldArray as $fieldName) {
            $fieldArray[] = $allFieldArray[$fieldName];
        }

        return $fieldArray;
    }

    public function buildFieldArrayForm(): array
    {
        $fieldArray = [];
        $panelArray = $this->configurePanels();

        if($panelArray && count($panelArray) > 0) {
            foreach($panelArray as $panel) {
                $fieldArray = array_merge(
                        $fieldArray,
                        $this->buildFieldArrayFormByPanel($panel)
                );
            }
        }
        else {
            $fieldArray = $this->buildFieldArray($this->configureForm());
        }

        return $fieldArray;
    }

    public function buildFieldArrayFormByPanel(string $panel): array
    {
        $method = 'configurePanel' . ucfirst($panel);
        if(method_exists($this, $method)) {
            $panelFieldArray = $this->$method();
        }
        else{
            throw new \Exception($method . ' n\'existe pas ');
        }

        return array_merge(
                ['panel_' . $panel => FormField::addPanel($panel)],
                $this->buildFieldArray($panelFieldArray)
        );
    }

}